@@ -27,7 +27,6 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 
-#include <assert.h>
 #include <errno.h>
 #include <stdlib.h>
 #include <unistd.h>
@@ -35,9 +34,9 @@
 #include <dpkg/i18n.h>
 #include <dpkg/dpkg.h>
 #include <dpkg/dpkg-db.h>
+#include <dpkg/db-ctrl.h>
+#include <dpkg/db-fsys.h>
 
-#include "filesdb.h"
-#include "infodb.h"
 #include "main.h"
 
 struct deppossi_pkg_iterator {
@@ -76,7 +75,8 @@ deppossi_pkg_iter_next(struct deppossi_pkg_iterator *iter)
       pkgbin = &pkg_cur->available;
       break;
     case wpb_by_istobe:
-      if (pkg_cur->clientdata->istobe == PKG_ISTOBE_INSTALLNEW)
+      if (pkg_cur->clientdata &&
+          pkg_cur->clientdata->istobe == PKG_ISTOBE_INSTALLNEW)
         pkgbin = &pkg_cur->available;
       else
         pkgbin = &pkg_cur->installed;
@@ -224,54 +224,59 @@ findbreakcyclerecursive(struct pkginfo *pkg, struct cyclesofarlink *sofar)
 bool
 findbreakcycle(struct pkginfo *pkg)
 {
-  struct pkgiterator *iter;
+  struct pkg_hash_iter *iter;
   struct pkginfo *tpkg;
 
   /* Clear the visited flag of all packages before we traverse them. */
-  iter = pkg_db_iter_new();
-  while ((tpkg = pkg_db_iter_next_pkg(iter))) {
+  iter = pkg_hash_iter_new();
+  while ((tpkg = pkg_hash_iter_next_pkg(iter))) {
     ensure_package_clientdata(tpkg);
     tpkg->clientdata->color = PKG_CYCLE_WHITE;
   }
-  pkg_db_iter_free(iter);
+  pkg_hash_iter_free(iter);
 
   return findbreakcyclerecursive(pkg, NULL);
 }
 
 void describedepcon(struct varbuf *addto, struct dependency *dep) {
-  const char *fmt;
   struct varbuf depstr = VARBUF_INIT;
 
+  varbufdependency(&depstr, dep);
+  varbuf_end_str(&depstr);
+
   switch (dep->type) {
   case dep_depends:
-    fmt = _("%s depends on %s");
+    varbuf_printf(addto, _("%s depends on %s"),
+                  pkg_name(dep->up, pnaw_nonambig), depstr.buf);
     break;
   case dep_predepends:
-    fmt = _("%s pre-depends on %s");
+    varbuf_printf(addto, _("%s pre-depends on %s"),
+                  pkg_name(dep->up, pnaw_nonambig), depstr.buf);
     break;
   case dep_recommends:
-    fmt = _("%s recommends %s");
+    varbuf_printf(addto, _("%s recommends %s"),
+                  pkg_name(dep->up, pnaw_nonambig), depstr.buf);
     break;
   case dep_suggests:
-    fmt = _("%s suggests %s");
+    varbuf_printf(addto, _("%s suggests %s"),
+                  pkg_name(dep->up, pnaw_nonambig), depstr.buf);
     break;
   case dep_breaks:
-    fmt = _("%s breaks %s");
+    varbuf_printf(addto, _("%s breaks %s"),
+                  pkg_name(dep->up, pnaw_nonambig), depstr.buf);
     break;
   case dep_conflicts:
-    fmt = _("%s conflicts with %s");
+    varbuf_printf(addto, _("%s conflicts with %s"),
+                  pkg_name(dep->up, pnaw_nonambig), depstr.buf);
     break;
   case dep_enhances:
-    fmt = _("%s enhances %s");
+    varbuf_printf(addto, _("%s enhances %s"),
+                  pkg_name(dep->up, pnaw_nonambig), depstr.buf);
     break;
   default:
     internerr("unknown deptype '%d'", dep->type);
   }
 
-  varbufdependency(&depstr, dep);
-  varbuf_end_str(&depstr);
-
-  varbuf_printf(addto, fmt, pkg_name(dep->up, pnaw_nonambig), depstr.buf);
   varbuf_destroy(&depstr);
 }
 
@@ -314,10 +319,14 @@ depisok(struct dependency *dep, struct varbuf *whynot,
    * Allow 250x3 for package names, versions, &c, + 250 for ourselves. */
   char linebuf[1024];
 
-  assert(dep->type == dep_depends || dep->type == dep_predepends ||
-	 dep->type == dep_breaks || dep->type == dep_conflicts ||
-	 dep->type == dep_recommends || dep->type == dep_suggests ||
-	 dep->type == dep_enhances);
+  if (dep->type != dep_depends &&
+      dep->type != dep_predepends &&
+      dep->type != dep_breaks &&
+      dep->type != dep_conflicts &&
+      dep->type != dep_recommends &&
+      dep->type != dep_suggests &&
+      dep->type != dep_enhances)
+    internerr("unknown dependency type %d", dep->type);
 
   if (canfixbyremove)
     *canfixbyremove = NULL;
@@ -331,7 +340,7 @@ depisok(struct dependency *dep, struct varbuf *whynot,
   case PKG_ISTOBE_DECONFIGURE:
     return true;
   case PKG_ISTOBE_NORMAL:
-    /* Only installed packages can be make dependency problems. */
+    /* Only installed packages can be made dependency problems. */
     switch (dep->up->status) {
     case PKG_STAT_INSTALLED:
     case PKG_STAT_TRIGGERSPENDING:
@@ -420,8 +429,7 @@ depisok(struct dependency *dep, struct varbuf *whynot,
           case PKG_STAT_TRIGGERSAWAITED:
               if (canfixbytrigaw && versionsatisfied(&pkg_pos->installed, possi))
                 *canfixbytrigaw = pkg_pos;
-              /* Fall through to have a chance to return OK due to
-               * allowunconfigd and to fill the explanation */
+              /* Fall through. */
           case PKG_STAT_UNPACKED:
           case PKG_STAT_HALFCONFIGURED:
             if (allowunconfigd) {
@@ -581,6 +589,7 @@ depisok(struct dependency *dep, struct varbuf *whynot,
           case PKG_STAT_HALFCONFIGURED:
             if (dep->type == dep_breaks)
               break; /* No problem. */
+            /* Fall through. */
           case PKG_STAT_INSTALLED:
           case PKG_STAT_TRIGGERSPENDING:
           case PKG_STAT_TRIGGERSAWAITED:
@@ -651,6 +660,7 @@ depisok(struct dependency *dep, struct varbuf *whynot,
         case PKG_ISTOBE_DECONFIGURE:
           if (dep->type == dep_breaks)
             continue; /* Already deconfiguring. */
+          /* Fall through. */
         case PKG_ISTOBE_NORMAL:
         case PKG_ISTOBE_PREINSTALL:
           switch (provider->up->up->status) {
@@ -662,6 +672,7 @@ depisok(struct dependency *dep, struct varbuf *whynot,
           case PKG_STAT_HALFCONFIGURED:
             if (dep->type == dep_breaks)
               break; /* No problem. */
+            /* Fall through. */
           case PKG_STAT_INSTALLED:
           case PKG_STAT_TRIGGERSPENDING:
           case PKG_STAT_TRIGGERSAWAITED:
